@@ -641,6 +641,16 @@ is_laptop (MateRRScreen *screen, MateRROutputInfo *output)
         return mate_rr_output_is_laptop (rr_output);
 }
 
+static gboolean
+is_primary (MateRRScreen *screen, MateRROutputInfo *output)
+{
+        MateRROutput *rr_output;
+
+        rr_output = mate_rr_screen_get_output_by_name (screen, mate_rr_output_info_get_name (output));
+        return mate_rr_output_get_is_primary(rr_output);
+}
+
+
 static gboolean
 get_clone_size (MateRRScreen *screen, int *width, int *height)
 {
@@ -758,9 +768,7 @@ make_clone_setup (MateRRScreen *screen)
                                 h = mate_rr_mode_get_height (mode);
 
                                 if (w == width && h == height) {
-                                        int r = mate_rr_mode_get_freq (mode);
-                                        if (r > best_rate)
-                                                best_rate = r;
+                                        best_rate = mate_rr_mode_get_freq (mode);
                                 }
                         }
 
@@ -851,6 +859,35 @@ turn_on (MateRRScreen *screen,
         return FALSE;
 }
 
+static MateRRConfig *
+make_primary_only_setup (MateRRScreen *screen)
+{
+        /*Leave all of the monitors turned on, just change from mirror to xinerama layout*/
+        MateRRConfig *result = mate_rr_config_new_current (screen, NULL);
+        MateRROutputInfo **outputs = mate_rr_config_get_outputs (result);
+        int i, x, width,height;
+        x = 0;
+
+        for (i = 0; outputs[i] != NULL; ++i) {
+                MateRROutputInfo *info = outputs[i];
+                width = mate_rr_output_info_get_preferred_width (info);
+                height = mate_rr_output_info_get_preferred_height (info);
+                mate_rr_output_info_set_geometry (info, x, 0, width, height);
+                mate_rr_output_info_set_active (info, TRUE);
+                x = x + width;
+        }
+
+        if (result && config_is_all_off (result)) {
+                g_object_unref (G_OBJECT (result));
+                result = NULL;
+        }
+
+        mate_rr_config_set_clone (result, FALSE);
+        print_configuration (result, "Primary only setup");
+
+        return result;
+}
+
 static MateRRConfig *
 make_laptop_setup (MateRRScreen *screen)
 {
@@ -1672,76 +1709,6 @@ status_icon_popup_menu_selection_done_cb (GtkMenuShell *menu_shell, gpointer dat
 #define OUTPUT_TITLE_ITEM_BORDER 2
 #define OUTPUT_TITLE_ITEM_PADDING 4
 
-/* This is an expose-event hander for the title label for each MateRROutput.
- * We want each title to have a colored background, so we paint that background, then
- * return FALSE to let GtkLabel expose itself (i.e. paint the label's text), and then
- * we have a signal_connect_after handler as well.  See the comments below
- * to see why that "after" handler is needed.
- */
-static gboolean
-output_title_label_draw_cb (GtkWidget *widget, cairo_t *cr, gpointer data)
-{
-        MsdXrandrManager *manager = MSD_XRANDR_MANAGER (data);
-        struct MsdXrandrManagerPrivate *priv = manager->priv;
-        MateRROutputInfo *output;
-        GdkRGBA color;
-        GtkAllocation allocation;
-
-        g_assert (GTK_IS_LABEL (widget));
-
-        output = g_object_get_data (G_OBJECT (widget), "output");
-        g_assert (output != NULL);
-
-        g_assert (priv->labeler != NULL);
-
-        /* Draw a black rectangular border, filled with the color that corresponds to this output */
-        mate_rr_labeler_get_rgba_for_output (priv->labeler, output, &color);
-
-        cairo_set_source_rgb (cr, 0, 0, 0);
-        cairo_set_line_width (cr, OUTPUT_TITLE_ITEM_BORDER);
-        gtk_widget_get_allocation (widget, &allocation);
-        cairo_rectangle (cr,
-                         allocation.x + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         allocation.y + OUTPUT_TITLE_ITEM_BORDER / 2.0,
-                         allocation.width - OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.height - OUTPUT_TITLE_ITEM_BORDER);
-        cairo_stroke (cr);
-
-        gdk_cairo_set_source_rgba (cr, &color);
-        cairo_rectangle (cr,
-                         allocation.x + OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.y + OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.width - 2 * OUTPUT_TITLE_ITEM_BORDER,
-                         allocation.height - 2 * OUTPUT_TITLE_ITEM_BORDER);
-
-        cairo_fill (cr);
-
-        /* We want the label to always show up as if it were sensitive
-         * ("style->fg[GTK_STATE_NORMAL]"), even though the label is insensitive
-         * due to being inside an insensitive menu item.  So, here we have a
-         * HACK in which we frob the label's state directly.  GtkLabel's expose
-         * handler will be run after this function, so it will think that the
-         * label is in GTK_STATE_NORMAL.  We reset the label's state back to
-         * insensitive in output_title_label_after_expose_event_cb().
-         *
-         * Yay for fucking with GTK+'s internals.
-         */
-
-        gtk_widget_set_state (widget, GTK_STATE_NORMAL);
-
-        return FALSE;
-}
-
-/* See the comment in output_title_event_box_expose_event_cb() about this funny label widget */
-static gboolean
-output_title_label_after_draw_cb (GtkWidget *widget, cairo_t *cr, gpointer data)
-{
-        g_assert (GTK_IS_LABEL (widget));
-        gtk_widget_set_state (widget, GTK_STATE_INSENSITIVE);
-
-        return FALSE;
-}
-
 static void
 title_item_size_allocate_cb (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
 {
@@ -1784,12 +1751,26 @@ title_item_size_allocate_cb (GtkWidget *widget, GtkAllocation *allocation, gpoin
 static GtkWidget *
 make_menu_item_for_output_title (MsdXrandrManager *manager, MateRROutputInfo *output)
 {
-        GtkWidget *item;
-        GtkWidget *label;
+        GtkWidget       *item;
+        GtkStyleContext *context;
+        GtkCssProvider  *provider, *provider2;
+        GtkWidget       *label;
+        GtkWidget       *image;
+        GtkWidget *box;
         char *str;
-        GdkColor black = { 0, 0, 0, 0 };
+        GString *string;
+        GdkRGBA color;
+        gchar *css, *color_string, *theme_name;
+        GtkSettings *settings;
+        GSettings *icon_settings;
+
+        struct MsdXrandrManagerPrivate *priv = manager->priv;
 
         item = gtk_menu_item_new ();
+        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+        image = gtk_image_new_from_icon_name ("computer", GTK_ICON_SIZE_MENU);
+        context = gtk_widget_get_style_context (item);
+        gtk_style_context_add_class (context, "xrandr-applet");
 
         g_signal_connect (item, "size-allocate",
                           G_CALLBACK (title_item_size_allocate_cb), NULL);
@@ -1799,38 +1780,118 @@ make_menu_item_for_output_title (MsdXrandrManager *manager, MateRROutputInfo *ou
         gtk_label_set_markup (GTK_LABEL (label), str);
         g_free (str);
 
-        /* Make the label explicitly black.  We don't want it to follow the
-         * theme's colors, since the label is always shown against a light
-         * pastel background.  See bgo#556050
-         */
-        gtk_widget_modify_fg (label, gtk_widget_get_state (label), &black);
-
         /* Add padding around the label to fit the box that we'll draw for color-coding */
-#if GTK_CHECK_VERSION (3, 16, 0)
         gtk_label_set_xalign (GTK_LABEL (label), 0.0);
         gtk_label_set_yalign (GTK_LABEL (label), 0.5);
-#else
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-#endif
         gtk_widget_set_margin_start (label, OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING);
         gtk_widget_set_margin_end (label, OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING);
         gtk_widget_set_margin_top (label, OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING);
         gtk_widget_set_margin_bottom (label, OUTPUT_TITLE_ITEM_BORDER + OUTPUT_TITLE_ITEM_PADDING);
 
-        gtk_container_add (GTK_CONTAINER (item), label);
+        /*Load the icon unless the user has icons in menus turned off*/
+        icon_settings = g_settings_new ("org.mate.interface");
+        if (g_settings_get_boolean (icon_settings, "menus-have-icons")){
+            gtk_container_add (GTK_CONTAINER (box), image);
+            }
+        gtk_container_add (GTK_CONTAINER (box), label);
+        gtk_container_add (GTK_CONTAINER (item), box);
 
-        /* We want to paint a colored box as the background of the label, so we connect
-         * to its expose-event signal.  See the comment in *** to see why need to connect
-         * to the label both 'before' and 'after'.
+        mate_rr_labeler_get_rgba_for_output (priv->labeler, output, &color);
+
+        color_string = gdk_rgba_to_string (&color);
+
+        /*This can be overriden by themes, check all label:insensitive entries if it does not show up*/
+        string = g_string_new(NULL);
+        g_string_append (string, ".mate-panel-menu-bar menuitem.xrandr-applet:disabled>box>label{\n");
+        /*g_string_append (string, "color: black;"); Does not work-overridden in all themes*/
+        g_string_append (string, "padding-left: 4px; padding-right: 4px;");
+        g_string_append (string, "border-color: gray;");
+        g_string_append (string, "border-width: 1px;");
+        g_string_append (string, "border-style: inset;");
+        g_string_append (string, "background-image: none;");
+        /*Bright color for active monitor, dimmed for inactive monitor*/
+        if (mate_rr_output_info_is_active (output)){
+                g_string_append (string, "background-color:");
+                g_string_append (string, color_string);
+                g_string_append (string, ";");
+                g_string_append (string," }");
+        }
+        else{
+                g_string_append (string, "background-color: alpha(");
+                g_string_append (string, color_string);
+                g_string_append (string, ", 0.4);");
+                g_string_append (string," }");
+                gtk_style_context_add_class (context, "monitor-off");
+        }
+
+        css = g_string_free (string, FALSE);
+
+        context = gtk_widget_get_style_context (label);
+        provider = gtk_css_provider_new ();
+        gtk_css_provider_load_from_data (provider,css, -1, NULL);
+
+        gtk_style_context_add_provider (context,
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_FALLBACK);
+
+        g_object_unref (provider);
+        g_free (color_string);
+        g_free (css);
+
+        /*This is NOT overrridden by themes as FALLBACK won't work here
+         *Disable dim/opacity effects applied to icons in an insensitive menu item
          */
-        g_signal_connect (label, "draw",
-                          G_CALLBACK (output_title_label_draw_cb), manager);
-        g_signal_connect_after (label, "draw",
-                                G_CALLBACK (output_title_label_after_draw_cb) , manager);
 
-        g_object_set_data (G_OBJECT (label), "output", output);
+        context = gtk_widget_get_style_context (image);
+        provider = gtk_css_provider_new ();
+
+        gtk_css_provider_load_from_data (provider,
+            ".mate-panel-menu-bar menuitem.xrandr-applet:disabled>box>image{\n"
+             "opacity: 1.0; \n"
+             "-gtk-icon-style:regular; \n" /* symbolic icons would get the disabled color*/
+             "-gtk-icon-effect: none; \n"
+             "}",
+             -1, NULL);
+        gtk_style_context_add_provider (context,
+					GTK_STYLE_PROVIDER (provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+        /*Deal with the GNOME and *bird themes, match display capplet theme */
+        provider2 = gtk_css_provider_new ();
+        settings = gtk_settings_get_default();
+        context = gtk_widget_get_style_context (label);
+        g_object_get (settings, "gtk-theme-name", &theme_name, NULL);
+        if (g_strcmp0 (theme_name, "Adwaita") == 0 ||
+                      g_strcmp0 (theme_name, "Adwaita-dark") == 0 ||
+                      g_strcmp0 (theme_name, "Raleigh") == 0 ||
+                      g_strcmp0 (theme_name, "win32") == 0 ||
+                      g_strcmp0 (theme_name, "HighContrast") == 0 ||
+                      g_strcmp0 (theme_name, "HighContrastInverse") == 0 ||
+                      g_strcmp0 (theme_name, "Blackbird") == 0 ||
+                      g_strcmp0 (theme_name, "Bluebird") == 0 ||
+                      g_strcmp0 (theme_name, "Greybird") == 0){
+            gtk_css_provider_load_from_data (provider2,
+                    ".mate-panel-menu-bar menuitem.xrandr-applet:disabled>box>label{\n"
+                    "color: black;\n"
+                    "}"
+                    ".mate-panel-menu-bar menuitem.xrandr-applet.monitor-off:disabled>box>label{\n"
+                     "color: alpha (black, 0.6);\n"
+                    "}",
+                    -1, NULL);
+            gtk_style_context_add_provider(context,
+					    GTK_STYLE_PROVIDER (provider2),
+					    GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        }
+        /*Keep or take this off all other themes as soon as the theme changes*/
+        else{
+            gtk_style_context_remove_provider(context, GTK_STYLE_PROVIDER (provider2));
+        }
+
+        g_object_unref (provider);
+        g_object_unref (provider2);
 
         gtk_widget_set_sensitive (item, FALSE); /* the title is not selectable */
+
         gtk_widget_show_all (item);
 
         return item;
@@ -1920,6 +1981,47 @@ ensure_current_configuration_is_saved (void)
         g_object_unref (rr_screen);
 }
 
+static void
+monitor_activate_cb (GtkCheckMenuItem *item, gpointer data)
+{
+        MsdXrandrManager *manager = MSD_XRANDR_MANAGER (data);
+        struct MsdXrandrManagerPrivate *priv = manager->priv;
+        MateRROutputInfo *output;
+        GError *error;
+
+        ensure_current_configuration_is_saved ();
+
+        output = g_object_get_data (G_OBJECT (item), "output");
+
+        /*This is borrowed from the capplet in mate-control-center
+         *And shares the same limitations concerning monitors
+         *which have been turned off and back on without being reconfigured
+         */
+        if (gtk_check_menu_item_get_active (item)){
+                int x, y, width, height;
+                mate_rr_output_info_get_geometry (output, &x, &y, NULL, NULL);
+                width = mate_rr_output_info_get_preferred_width (output);
+                height = mate_rr_output_info_get_preferred_height (output);
+                mate_rr_output_info_set_geometry (output, x, y, width, height);
+                mate_rr_output_info_set_active (output, TRUE);
+
+        }
+        else{
+                mate_rr_output_info_set_active (output, FALSE);
+        }
+
+        error = NULL;
+        if (!mate_rr_config_save (priv->configuration, &error)) {
+                error_message (manager, _("Could not save monitor configuration"), error, NULL);
+                if (error)
+                        g_error_free (error);
+
+                return;
+        }
+
+        try_to_apply_intended_configuration (manager, NULL, gtk_get_current_event_time (), NULL); 
+}
+
 static void
 output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
 {
@@ -1952,6 +2054,41 @@ output_rotation_item_activate_cb (GtkCheckMenuItem *item, gpointer data)
         try_to_apply_intended_configuration (manager, NULL, gtk_get_current_event_time (), NULL); /* NULL-GError */
 }
 
+static void
+mirror_outputs_cb(GtkCheckMenuItem *item, gpointer data)
+{
+        MsdXrandrManager *manager = MSD_XRANDR_MANAGER (data);
+        struct MsdXrandrManagerPrivate *priv = manager->priv;
+        MateRRScreen *screen = priv->rw_screen;
+
+        if (gtk_check_menu_item_get_active(item)){
+
+                MateRRConfig *config;
+                config = make_clone_setup (screen);
+                if (!config || config == NULL){
+                        error_message (manager, _("Mirroring outputs not supported"), NULL, NULL);
+                }
+
+                mate_rr_config_save (config, NULL);
+                try_to_apply_intended_configuration (manager, NULL, gtk_get_current_event_time (), NULL); 
+
+                g_object_unref (config);
+
+        }
+        else{
+
+                MateRRConfig *config;
+                config = make_primary_only_setup (screen);
+                /*If nothing worked, bring up the display capplet so the user can reconfigure*/
+                if (config == NULL)
+                         run_display_capplet(GTK_WIDGET(item));
+                mate_rr_config_save (config, NULL);
+                try_to_apply_intended_configuration (manager, NULL, gtk_get_current_event_time (), NULL); 
+
+                g_object_unref (config);
+
+        }
+}
 static void
 add_items_for_rotations (MsdXrandrManager *manager, MateRROutputInfo *output, MateRRRotation allowed_rotations)
 {
@@ -1993,6 +2130,10 @@ add_items_for_rotations (MsdXrandrManager *manager, MateRROutputInfo *output, Ma
                 }
 
                 item = gtk_radio_menu_item_new_with_label (group, _(rotations[i].name));
+                /*HERE*/
+                if (!(mate_rr_output_info_is_active (output))){
+                    gtk_widget_set_sensitive (item, FALSE); /*Rotation can't be set from the OFF state*/
+                }
                 gtk_widget_show_all (item);
                 gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 
@@ -2038,6 +2179,47 @@ add_rotation_items_for_output (MsdXrandrManager *manager, MateRROutputInfo *outp
                 add_items_for_rotations (manager, output, rotations);
 }
 
+static void
+add_enable_option_for_output (MsdXrandrManager *manager, MateRROutputInfo *output)
+{
+        struct MsdXrandrManagerPrivate *priv = manager->priv; 
+        GtkWidget *item;
+        gulong activate_id;
+
+        item = gtk_check_menu_item_new();
+
+        if (mate_rr_output_info_is_active (output)){
+                gtk_menu_item_set_label (GTK_MENU_ITEM(item), "ON");
+                gtk_widget_set_tooltip_text(item, "Turn this monitor off");
+        }
+        else {
+                gtk_menu_item_set_label (GTK_MENU_ITEM(item), "OFF");
+                gtk_widget_set_tooltip_text(item ,"Turn this monitor on");
+        }
+
+        gtk_widget_show_all (item);
+        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
+
+        g_object_set_data (G_OBJECT (item), "output", output);
+
+        activate_id = g_signal_connect (item, "activate",
+                                G_CALLBACK (monitor_activate_cb), manager);
+
+        /* Block the signal temporarily so our callback won't be called;
+        * we are just setting up the UI.
+        */
+        g_signal_handler_block (item, activate_id);
+
+        if (mate_rr_output_info_is_active (output)){
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+        }
+        else{
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), FALSE);
+        }
+
+        g_signal_handler_unblock (item, activate_id);
+}
+
 static void
 add_menu_items_for_output (MsdXrandrManager *manager, MateRROutputInfo *output)
 {
@@ -2047,6 +2229,7 @@ add_menu_items_for_output (MsdXrandrManager *manager, MateRROutputInfo *output)
         item = make_menu_item_for_output_title (manager, output);
         gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 
+        add_enable_option_for_output (manager, output);
         add_rotation_items_for_output (manager, output);
 }
 
@@ -2064,11 +2247,40 @@ add_menu_items_for_outputs (MsdXrandrManager *manager)
         }
 }
 
+static void
+add_menu_items_for_clone (MsdXrandrManager *manager)
+{
+        struct MsdXrandrManagerPrivate *priv = manager->priv;
+        GtkWidget *item;
+        gulong activate_id;
+
+        item = gtk_check_menu_item_new_with_label("Same output all monitors");
+        gtk_widget_set_tooltip_text(item, "Mirror same output to all monitors and turn them on");        
+        gtk_widget_show_all (item);
+        gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
+        activate_id =  g_signal_connect (item, "activate",
+                                G_CALLBACK (mirror_outputs_cb), manager);
+        /*Block the handler until the GUI is set up no matter what the monitor state*/
+        g_signal_handler_block (item, activate_id);
+
+        if (mate_rr_config_get_clone(priv->configuration)){
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+        }
+        else{
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), FALSE);
+        }
+        g_signal_handler_unblock (item, activate_id);
+}
+
 static void
 status_icon_popup_menu (MsdXrandrManager *manager, guint button, guint32 timestamp)
 {
         struct MsdXrandrManagerPrivate *priv = manager->priv;
         GtkWidget *item;
+        GtkWidget *image;
+        GtkWidget *label;
+        GtkWidget *box;
+        GSettings *icon_settings;
 
         g_assert (priv->configuration == NULL);
         priv->configuration = mate_rr_config_new_current (priv->rw_screen, NULL);
@@ -2085,10 +2297,25 @@ status_icon_popup_menu (MsdXrandrManager *manager, guint button, guint32 timesta
         gtk_widget_show (item);
         gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 
-        item = gtk_menu_item_new_with_mnemonic (_("_Configure Display Settings…"));
+        add_menu_items_for_clone (manager);
+
+        item = gtk_menu_item_new();
+        box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 10);
+        image = gtk_image_new_from_icon_name ("preferences-system", GTK_ICON_SIZE_MENU);
+        label = gtk_label_new_with_mnemonic("_Configure Display Settings…");
+        /*Load the icon unless the user has icons in menus turned off*/
+        icon_settings = g_settings_new ("org.mate.interface");
+        if (g_settings_get_boolean (icon_settings, "menus-have-icons")){
+            gtk_container_add (GTK_CONTAINER (box), image);
+            g_signal_connect (item, "size-allocate",
+                          G_CALLBACK (title_item_size_allocate_cb), NULL);
+            }
+        gtk_container_add (GTK_CONTAINER (box), label);
+        gtk_container_add (GTK_CONTAINER (item), box);
+        gtk_widget_set_tooltip_text(item, "Open the display configuration dialog (all settings)");
         g_signal_connect (item, "activate",
                           G_CALLBACK (popup_menu_configure_display_cb), manager);
-        gtk_widget_show (item);
+        gtk_widget_show_all (item);
         gtk_menu_shell_append (GTK_MENU_SHELL (priv->popup_menu), item);
 
         g_signal_connect (priv->popup_menu, "selection-done",
